--
-- QWOP-like by knarf <k@b0.cx>
-- 

-- box2D new box shape+body
function box(x, y, w, h, m, r)
   local o = {}
   r = r or 0
   x = x + w/2
   y = y + h/2
   o.body  = love.physics.newBody(world, x, y, m, r)
   o.shape = love.physics.newRectangleShape(o.body, 0, 0, w, h)
   o.color = {math.random(256), math.random(256), math.random(256)}
   table.insert(drawlist, o)
   return o
end

-- draw box
function dr(s)
   love.graphics.setColor(s.color[1], s.color[2], s.color[3])
   love.graphics.polygon("fill", s.shape:getPoints())
end

function love.load()
   width = 650
   height = 650

   world = love.physics.newWorld(-30, -30, width, height)
   world:setGravity(0, 30)
   world:setMeter(64)
   drawlist = {}
   won = false
   started = false

   ground = box(0, height-20, width, 20, 0)
   local b = box(0, 0, 20, height-20, 0)
   b.shape:setCategory(2)
   b = box(width-20, 0, 20, height-20, 0)
   b.shape:setCategory(2)

   height = height+80
   top = height

   
   torso = box(width/2, height/2, 20, 100, 10, 5)
   front = box(width/2+2.5, height/2+80, 15, 80, 15, 2)
   back  = box(width/2+2.5, height/2+80, 15, 80, 15, .8)
   fcalf = box(width/2+2.5, height/2+160, 15, 50, 10, 5)
   bcalf = box(width/2+2.5, height/2+160, 15, 50, 10, 5)
   
   torso.shape:setFriction(.4)
   front.shape:setFriction(.4)
   back.shape:setFriction(.4)
   fcalf.shape:setFriction(.4)
   bcalf.shape:setFriction(.4)

   -- 2 joints for each leg
   ftjoint = love.physics.newRevoluteJoint(front.body, torso.body, 
					   width/2+10, height/2+90)
   btjoint = love.physics.newRevoluteJoint(back.body,  torso.body,
					   width/2+10, height/2+90)

   fljoint = love.physics.newRevoluteJoint(fcalf.body, front.body, 
					   width/2+10, height/2+160)
   bljoint = love.physics.newRevoluteJoint(bcalf.body, back.body,
					   width/2+10, height/2+160)
   height = height-80

   -- suppress some collision
   ftjoint:setCollideConnected(false)
   btjoint:setCollideConnected(false)
   fljoint:setCollideConnected(false)
   bljoint:setCollideConnected(false)
   
   ground.shape:setCategory(2)

   front.shape:setCategory(1)
   back.shape:setCategory(1)
   fcalf.shape:setCategory(1)
   bcalf.shape:setCategory(1)
   
   front.shape:setMask(1)
   back.shape:setMask(1)
   fcalf.shape:setMask(1)
   bcalf.shape:setMask(1)
   
   -- limit rotation of legs
   ftjoint:setLimits(-math.pi/2+.3, math.pi/2-.3)
   btjoint:setLimits(-math.pi/2+.3, math.pi/2-.3)
   fljoint:setLimits(-math.pi+.3, -.3)
   bljoint:setLimits(-math.pi+.3, -.3)

   ftjoint:setLimitsEnabled(true)
   btjoint:setLimitsEnabled(true)
   fljoint:setLimitsEnabled(true)
   bljoint:setLimitsEnabled(true)

   love.graphics.setMode(width, height, false, true, 0)
end

function maxheight(s)
   local p = {s.shape:getPoints()}
   local max = p[2]
   
   for i = 2, #p, 2 do
      if p[i] < max then 
	 max = p[i]
      end
   end
   return max   
end

function reset()
   torso.body:setX(width/2)
   torso.body:setY(height/2+200)
   top = height
end

function love.quit()
   return false
end

function love.keypressed(key, unicode)

   started = true

   if key == 'escape' then
      love.event.push("q")

   elseif key == 'r' then
      reset()
      end
   
end


function love.update(dt)
   if started then world:update(dt) end

   top = math.min(top, maxheight(torso),
		  maxheight(front),
		  maxheight(back),
		  maxheight(fcalf),
		  maxheight(bcalf))

   local a = 100 -- thighs
   local b = 200  -- calves

   if height-top > 300 and not won then
      win()
      won = true
   end

   if love.keyboard.isDown('q') then
      front.body:applyTorque(a)
      back.body:applyTorque(-a)
      
   elseif love.keyboard.isDown('w') then
      front.body:applyTorque(-a)
      back.body:applyTorque(a)
      
   elseif love.keyboard.isDown('o') then
      fcalf.body:applyTorque(-b)
      bcalf.body:applyTorque(b)

   elseif love.keyboard.isDown('p') then
      fcalf.body:applyTorque(b)
      bcalf.body:applyTorque(-b)
   end
end

function win()
   for i = 1, 100 do
      local b = box(math.random(10, width-10), -10, 20, 10, 1, .1)
      b.shape:setCategory(3)
   end
   
end


function love.draw()
   for _, e in pairs(drawlist) do
      dr(e)
   end
   
   -- local scale = .5+(height-top)/height
   local scale = math.sin(16*love.timer.getTime())*.1+1
   local str = string.format("TOP %.1f m", (height-top)/100)

   love.graphics.setColor(255, 255, 255)
   love.graphics.print("QWOP: high jump style! Surprise over 3 m...\n"..
		       "Use Q, W, O and P (in qwerty "..
		       "layout) to control the thighs/calves.\n"..
		       "ESC to quit\n"..
		       "Good luck!", 30, 30)

   love.graphics.line(0, top, width, top)
   love.graphics.print(str, width/2-30*scale, top, 0, scale, scale)
   
end
